@@ -11,7 +11,6 @@ extern crate gtk_sys;
 #[allow(unused_imports)]
 #[macro_use] // only for tests
 extern crate maplit;
-extern crate regex;
 extern crate serde;
 extern crate xkbcommon;
 
@@ -19,18 +18,21 @@ extern crate xkbcommon;
 mod logging;
 
 mod action;
+mod animation;
 pub mod data;
 mod drawing;
+mod event_loop;
 pub mod float_ord;
 pub mod imservice;
 mod keyboard;
 mod layout;
 mod locale;
-mod locale_config;
+mod main;
 mod manager;
 mod outputs;
 mod popover;
 mod resources;
+mod state;
 mod style;
 mod submission;
 pub mod tests;
